#pragma once

#define CONTEXT_GLOBAL               ((IV)0)
#define CONTEXT_BLOCK_COMMENT        ((IV)1)
#define CONTEXT_CREATE_TABLE         ((IV)2)
#define CONTEXT_CREATE_TABLE_COLUMN  ((IV)3)
#define CONTEXT_INSERT_INTO          ((IV)4)
#define CONTEXT_INSERT_VALUES        ((IV)5)
